from nonebot.plugin import PluginMetadata

from .__main__ import *  # type:ignore

__version__ = "0.5.0b2"
__plugin_meta__ = PluginMetadata(
    name="BAWiki",
    description="碧蓝档案Wiki插件",
    usage="见下表",
    extra={
        "menu_data": [
            {
                "func": "日程表",
                "trigger_method": "指令",
                "trigger_condition": "ba日程表",
                "brief_des": "查看活动日程表",
                "detail_des": (
                    "查看当前未结束的卡池、活动以及起止时间，"
                    "默认为GameKee源，可以在指令后带参数使用SchaleDB数据源\n"
                    "指令示例：\n"
                    "<ft color=(238,120,0)>ba日程表</ft> （GameKee源）；\n"
                    "<ft color=(238,120,0)>ba日程表 schaledb</ft> （SchaleDB源，日服）；\n"
                    "<ft color=(238,120,0)>ba日程表 国际服</ft> （SchaleDB源，国际服）"
                ),
            },
            {
                "func": "学生图鉴",
                "trigger_method": "指令",
                "trigger_condition": "ba学生图鉴",
                "brief_des": "查询学生详情（SchaleDB）",
                "detail_des": (
                    "访问对应学生SchaleDB页面并截图，支持部分学生别名\n"
                    "指令示例："
                    "<ft color=(238,120,0)>ba学生图鉴 白子</ft>；"
                    "<ft color=(238,120,0)>ba学生图鉴 xcw</ft>"
                ),
            },
            {
                "func": "学生Wiki",
                "trigger_method": "指令",
                "trigger_condition": "ba学生wiki",
                "brief_des": "查询学生详情（GameKee）",
                "detail_des": (
                    "访问对应学生GameKee Wiki页面并截图，支持部分学生别名\n"
                    "可以用这些指令触发："
                    "<ft color=(238,120,0)>ba学生wiki</ft>；"
                    "<ft color=(238,120,0)>ba学生Wiki</ft>；"
                    "<ft color=(238,120,0)>ba学生WIKI</ft>\n"
                    "指令示例："
                    "<ft color=(238,120,0)>ba学生wiki 白子</ft>；"
                    "<ft color=(238,120,0)>ba学生wiki xcw</ft>"
                ),
            },
            {
                "func": "新学生导航",
                "trigger_method": "指令",
                "trigger_condition": "ba新学生",
                "brief_des": "快速查询当期UP学生",
                "detail_des": "快速查询各日服、国际服当前UP学生简介与国际服下期预测UP",
            },
            {
                "func": "Live2D预览",
                "trigger_method": "指令",
                "trigger_condition": "baL2D",
                "brief_des": "查看学生的Live2D看板",
                "detail_des": (
                    "查看指定学生的Live2D回忆大厅预览图（可能为gif），支持部分学生别名\n"
                    "可以用这些指令触发："
                    "<ft color=(238,120,0)>bal2d</ft>；"
                    "<ft color=(238,120,0)>baL2D</ft>；"
                    "<ft color=(238,120,0)>balive2d</ft>；"
                    "<ft color=(238,120,0)>baLive2D</ft>\n"
                    "指令示例："
                    "<ft color=(238,120,0)>bal2d xcw</ft>"
                ),
            },
            {
                "func": "羁绊查询",
                "trigger_method": "指令",
                "trigger_condition": "ba羁绊",
                "brief_des": "查询学生解锁L2D需求的羁绊等级",
                "detail_des": (
                    "使用学生名称查询该学生解锁L2D看板需求的羁绊等级以及L2D预览，"
                    "或者使用羁绊等级级数查询哪些学生达到该等级时解锁L2D看板\n"
                    "使用学生名称查询时支持部分学生别名\n"
                    "可以用这些指令触发："
                    "<ft color=(238,120,0)>ba羁绊</ft>；"
                    "<ft color=(238,120,0)>ba好感度</ft>\n"
                    "指令示例："
                    "<ft color=(238,120,0)>ba羁绊 xcw</ft>；"
                    "<ft color=(238,120,0)>ba羁绊 9</ft>"
                ),
            },
        ],
        "menu_template": "default",
    },
)



import asyncio
import re
import time
from datetime import datetime
from io import BytesIO
from typing import List

from aiohttp import ClientSession
from nonebot import logger
from nonebot_plugin_htmlrender import get_new_page
from nonebot_plugin_imageutils import BuildImage, text2image
from playwright.async_api import Page

from .const import EXTRA_L2D_LI, RES_CALENDER_BANNER


async def game_kee_request(url, **kwargs):
    async with ClientSession() as s:
        async with s.get(
            url, headers={"game-id": "0", "game-alias": "ba"}, **kwargs
        ) as r:
            ret = await r.json()
            if ret["code"] != 0:
                raise ConnectionError(ret["msg"])
            return ret["data"]


async def get_calender():
    ret = await game_kee_request("https://ba.gamekee.com/v1/wiki/index")

    for i in ret:
        if i["module"]["id"] == 12:
            li: list = i["list"]

            now = time.time()
            li = [x for x in li if (now < x["end_at"])]

            li.sort(key=lambda x: x["end_at"])
            li.sort(key=lambda x: x["importance"], reverse=True)
            return li


async def get_stu_li():
    ret = await game_kee_request("https://ba.gamekee.com/v1/wiki/entry")

    for i in ret["entry_list"]:
        if i["id"] == 23941:

            for ii in i["child"]:
                if ii["id"] == 49443:
                    return {x["name"]: x for x in ii["child"]}


async def get_stu_cid_li():
    return {x: y["content_id"] for x, y in (await get_stu_li()).items()}


def game_kee_page_url(sid):
    return f"https://ba.gamekee.com/{sid}.html"


async def get_game_kee_page(url):
    async with get_new_page() as page:  # type:Page
        await page.goto(url, timeout=60 * 1000)

        # 删掉header
        await page.add_script_tag(
            content='document.getElementsByClassName("wiki-header")'
            ".forEach((v)=>{v.remove()})"
        )

        # 展开折叠的语音
        folds = await page.query_selector_all('xpath=//div[@class="fold-table-btn"]')
        for i in folds:
            try:
                await i.click()
            except:
                pass

        return await (
            await page.query_selector('xpath=//div[@class="wiki-detail-body"]')
        ).screenshot()


async def get_calender_page(ret):
    now = datetime.now()

    async def draw(it: dict):
        _p = None
        if _p := it.get("picture"):
            try:
                async with ClientSession() as _s:
                    async with _s.get(f"https:{_p}") as _r:
                        _p = await _r.read()
                _p = BuildImage.open(BytesIO(_p)).resize_width(1290)
            except:
                logger.exception("下载日程表图片失败")

        begin = datetime.fromtimestamp(it["begin_at"])
        end = datetime.fromtimestamp(it["end_at"])
        started = begin <= now
        time_remain = (end if started else begin) - now

        mm, ss = divmod(time_remain.seconds, 60)
        hh, mm = divmod(mm, 60)
        dd = time_remain.days or 0

        title_p = text2image(
            f'[b]{it["title"]}[/b]', "#ffffff00", max_width=1290, fontsize=65
        )
        time_p = text2image(
            f"{begin} ~ {end}", "#ffffff00", max_width=1290, fontsize=40
        )
        desc_p = (
            text2image(
                desc.replace("<br>", ""),
                "#ffffff00",
                max_width=1290,
                fontsize=40,
            )
            if (desc := it["description"])
            else None
        )
        remain_p = text2image(
            f"剩余 [color=#fc6475]{dd}[/color] 天 [color=#fc6475]{hh}[/color] 时 "
            f"[color=#fc6475]{mm}[/color] 分 [color=#fc6475]{ss}[/color] 秒"
            f'{"结束" if started else "开始"}',
            "#ffffff00",
            max_width=1290,
            fontsize=50,
        )

        h = (
            100
            + (title_p.height + 25)
            + (time_p.height + 25)
            + (_p.height + 25 if _p else 0)
            + (desc_p.height + 25 if desc_p else 0)
            + remain_p.height
        )
        img = BuildImage.new("RGBA", (1400, h), (255, 255, 255, 70)).draw_rectangle(
            (0, 0, 10, h), "#fc6475" if it["importance"] else "#4acf75"
        )

        if not started:
            img.draw_rectangle((1250, 0, 1400, 60), "gray")
            img.draw_text((1250, 0, 1400, 60), "未开始", 50, fill="white")

        ii = 50
        img.paste(title_p, (60, ii), True)
        ii += title_p.height + 25
        img.paste(time_p, (60, ii), True)
        ii += time_p.height + 25
        if _p:
            img.paste(_p, (60, ii), True)
            ii += _p.height + 25
        if desc_p:
            img.paste(desc_p, (60, ii), True)
            ii += desc_p.height + 25
        img.paste(remain_p, (60, ii), True)

        return img

    pics: List[BuildImage] = await asyncio.gather(  # type: ignore
        *[draw(x) for x in ret]
    )
    bg = (
        BuildImage.new("RGBA", (1500, 200 + sum([x.height + 50 for x in pics])))
        .gradient_color((138, 213, 244), (251, 226, 229))
        .paste(BuildImage.open(RES_CALENDER_BANNER).resize((1500, 150)))
        .draw_text(
            (50, 0, 1480, 150),
            "活动日程",
            100,
            weight="bold",
            fill="#ffffff",
            halign="left",
        )
    )

    index = 200
    for p in pics:
        bg.paste(p, (50, index), True)
        index += p.height + 50

    return bg.save_jpg()


async def grab_l2d(cid):
    r: dict = await game_kee_request(f"https://ba.gamekee.com/v1/content/detail/{cid}")
    r: str = r["content"]

    i = r.find('<div class="input-wrapper">官方介绍</div>')
    i = r.find('class="slide-item" data-index="2"', i)
    ii = r.find('data-index="3"', i)

    r: str = r[i:ii]

    img = re.findall('data-real="([^"]*)"', r)

    return [f"https:{x}" for x in img]


async def get_l2d(stu_name):
    if r := EXTRA_L2D_LI.get(stu_name):
        return r

    return await grab_l2d((await get_stu_cid_li()).get(stu_name))



import datetime

from .const import STU_ALIAS, SUFFIX_ALIAS


def format_timestamp(t):
    return datetime.datetime.fromtimestamp(t).strftime("%Y-%m-%d %H:%M:%S")


def recover_alia(origin: str, alia_dict: dict[str, list[str]]):
    origin = origin.lower().strip()

    # 精确匹配
    for k, li in alia_dict.items():
        if origin in li or origin == k:
            return k

    # 匹配括号别名
    for k, li in alia_dict.items():
        if (p := k.find("（")) != -1:
            prefixes = [k[:p]] + li
            suffixes = [k[p + 1 : -1]]

            if a := SUFFIX_ALIAS.get(suffixes[0]):
                suffixes.extend(a)

            for s in suffixes:
                for p in prefixes:
                    if f"{p}{s}" == origin or f"{s}{p}" == origin:
                        return k

    # 没找到，模糊匹配
    for k, li in alia_dict.items():
        li = [k] + li
        for v in li:
            if (v in origin) or (origin in v):
                return k

    return origin


def recover_stu_alia(a):
    return recover_alia(a, STU_ALIAS)



from nonebot import logger, on_command
from nonebot.adapters.onebot.v11 import Message, MessageSegment
from nonebot.internal.matcher import Matcher
from nonebot.params import CommandArg

from .const import ORIGIN_SCHALE_URL
from .data_gamekee import (
    game_kee_page_url,
    get_calender,
    get_calender_page,
    get_game_kee_page,
    get_l2d,
    get_stu_cid_li,
)
from .data_schaledb import (
    draw_fav_li,
    schale_get_calender,
    schale_get_stu_dict,
    schale_get_stu_info,
)
from .util import recover_stu_alia


async def schale_calender(matcher: Matcher, server=1):
    await matcher.send(f"请稍等，正在截取SchaleDB首页日程表～\n" f"{ORIGIN_SCHALE_URL}")

    try:
        img = MessageSegment.image(await schale_get_calender(server))
    except:
        logger.exception("截取schale db页面出错 home")
        return await matcher.finish("截取页面出错，请检查后台输出")

    await matcher.finish(img)


async def game_kee_calender(matcher: Matcher):
    try:
        ret = await get_calender()
    except:
        logger.exception("获取日程表出错")
        return await matcher.finish("获取日程表出错，请检查后台输出")

    if not ret:
        return await matcher.finish("没有获取到数据")

    await matcher.send("正在绘制图片，请稍等")
    try:
        pic = await get_calender_page(ret)
    except:
        logger.exception("绘制日程表图片出错")
        return await matcher.finish("绘制日程表图片出错，请检查后台输出")

    await matcher.finish(MessageSegment.image(pic))


handler_calender = on_command("ba日程表")


@handler_calender.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg: str = arg.extract_plain_text()
    if "国际服" in arg:
        await schale_calender(matcher)
    elif "schaledb" in arg.lower():
        await schale_calender(matcher, 0)
    else:
        await game_kee_calender(matcher)


async def send_wiki_page(sid, matcher: Matcher):
    url = game_kee_page_url(sid)
    await matcher.send(f"请稍等，正在截取Wiki页面……\n{url}")

    try:
        img = await get_game_kee_page(url)
    except:
        logger.exception(f"截取wiki页面出错 {url}")
        return await matcher.finish("截取页面出错，请检查后台输出")

    await matcher.finish(MessageSegment.image(img))


stu_schale = on_command("ba学生图鉴")


@stu_schale.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称")

    try:
        ret = await schale_get_stu_dict()
    except:
        logger.exception("获取学生列表出错")
        return await matcher.finish("获取学生列表表出错，请检查后台输出")

    if not ret:
        return await matcher.finish("没有获取到学生列表数据")

    if not (data := ret.get(recover_stu_alia(arg))):
        return await matcher.finish("未找到该学生")

    stu_name = data["PathName"]
    await matcher.send(f"请稍等，正在截取SchaleDB页面～\n" f"{ORIGIN_SCHALE_URL}?chara={stu_name}")

    try:
        img = MessageSegment.image(await schale_get_stu_info(stu_name))
    except:
        logger.exception(f"截取schale db页面出错 chara={stu_name}")
        return await matcher.finish("截取页面出错，请检查后台输出")

    await matcher.finish(img)


stu_wiki = on_command("ba学生wiki", aliases={"ba学生Wiki", "ba学生WIKI"})


@stu_wiki.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称")

    try:
        ret = await get_stu_cid_li()
    except:
        logger.exception("获取学生列表出错")
        return await matcher.finish("获取学生列表表出错，请检查后台输出")

    if not ret:
        return await matcher.finish("没有获取到学生列表数据")

    if not (sid := ret.get(recover_stu_alia(arg))):
        return await matcher.finish("未找到该学生")

    await send_wiki_page(sid, matcher)


new_stu = on_command("ba新学生")


@new_stu.handle()
async def _(matcher: Matcher):
    await send_wiki_page(155684, matcher)


l2d = on_command("bal2d", aliases={"baL2D", "balive2d", "baLive2D"})


@l2d.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称")

    if not (li := await get_l2d(recover_stu_alia(arg))):
        await matcher.finish(
            "没找到该学生的L2D看板\n"
            "可能原因：\n"
            "- GameKee页面爬取不到角色L2D图片\n"
            "- GameKee和插件没有收录该学生的L2D\n"
            "- 该学生没有L2D\n"
            "- 不存在该学生"
        )
        return

    await matcher.finish(Message([MessageSegment.image(x) for x in li]))


fav = on_command("ba好感度", aliases={"ba羁绊"})


@fav.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    arg = arg.extract_plain_text().strip()
    if not arg:
        return await matcher.finish("请提供学生名称或所需的羁绊等级")

    # 好感度等级
    if arg.isdigit():
        arg = int(arg)
        if arg > 9:
            return await matcher.finish("学生解锁L2D最高只需要羁绊等级9")
        if arg < 1:
            return await matcher.finish("学生解锁L2D最低只需要羁绊等级1")

        try:
            p = await draw_fav_li(arg)
        except:
            logger.exception("绘制图片出错")
            return await matcher.finish("绘制图片出错，请检查后台输出")

        return await matcher.finish(p)

    # 学生名称
    arg = recover_stu_alia(arg)

    try:
        ret = await schale_get_stu_dict()
    except:
        logger.exception("获取学生列表出错")
        return await matcher.finish("获取学生列表表出错，请检查后台输出")

    if stu := ret.get(arg):
        if not (lvl := stu["MemoryLobby"]):
            return await matcher.finish("该学生没有L2D")

        im = MessageSegment.text(f'{stu["Name"]} 在羁绊等级 {lvl[0]} 时即可解锁L2D\nL2D预览：')
        if p := await get_l2d(arg):
            im += [MessageSegment.image(x) for x in p]
        else:
            im += (
                "没找到该学生的L2D看板\n"
                "可能原因：\n"
                "- GameKee页面爬取不到角色L2D图片\n"
                "- GameKee和插件没有收录该学生的L2D\n"
            )
        return await matcher.finish(im)

    return await matcher.finish("未找到学生")



import asyncio
import math
from io import BytesIO
from pathlib import Path

from PIL import Image, ImageDraw, ImageFont
from aiohttp import ClientSession
from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot_plugin_htmlrender import get_new_page
from nonebot_plugin_imageutils import BuildImage
from playwright.async_api import Page, ViewportSize

from .const import RES_SCHALE_BG, SCHALE_DB_DIFFERENT, SCHALE_URL

PAGE_KWARGS = {
    "is_mobile": True,
    "viewport": ViewportSize(width=767, height=800),
}


async def schale_get_stu_data():
    async with ClientSession() as c:
        async with c.get(f"{SCHALE_URL}data/cn/students.min.json") as r:
            return await r.json()


async def schale_get_stu_dict():
    ret = await schale_get_stu_data()
    data = {x["Name"].replace("(", "（").replace(")", "）"): x for x in ret}

    for schale, gamekee in SCHALE_DB_DIFFERENT.items():
        if schale in data:
            data[gamekee] = data[schale]
            del data[schale]

    return data


async def schale_get_stu_info(stu):
    async with get_new_page(**PAGE_KWARGS) as page:  # type:Page
        await page.goto(
            f"{SCHALE_URL}?chara={stu}", timeout=60 * 1000, wait_until="networkidle"
        )

        # 进度条拉最大
        await page.add_script_tag(content="utilStuSetAllProgressMax();")

        return await page.screenshot(full_page=True)


async def schale_get_calender(server=1):
    async with get_new_page(**PAGE_KWARGS) as page:  # type:Page
        await page.goto(
            SCHALE_URL,
            timeout=60 * 1000,
            wait_until="domcontentloaded"
            # html加载完成需要立马改服 commit事件太早
        )

        await page.add_script_tag(
            content=f"regionID={server};" "loadModule('home');"  # 改服  # 防止进入之前的模块
        )
        await page.wait_for_load_state("networkidle")

        return await (
            # 当时活动标题的上级节点
            await page.query_selector('xpath=//*[@id="ba-home-server-info"]/..')
        ).screenshot()


async def draw_fav_li(lvl):
    try:
        stu_li = [
            x
            for x in await schale_get_stu_data()
            if (x["MemoryLobby"] and x["MemoryLobby"][0] == lvl)
        ]
    except:
        logger.exception("获取schale db学生数据失败")
        return "获取SchaleDB学生数据失败，请检查后台输出"

    if not stu_li:
        return f"没有学生在羁绊等级{lvl}时解锁L2D"

    txt_h = 48
    pic_h = 144
    icon_w = 182
    icon_h = pic_h + txt_h
    line_max_icon = 6

    if (l := len(stu_li)) <= line_max_icon:
        line = 1
        length = l
    else:
        line = math.ceil(l / line_max_icon)
        length = line_max_icon

    img = BuildImage.open(RES_SCHALE_BG).resize(
        (icon_w * length, icon_h * line + 5), keep_ratio=True
    )

    async def draw_stu(name_, dev_name_, line_, index_):
        left = index_ * icon_w
        top = line_ * icon_h + 5
        async with ClientSession() as s:
            async with s.get(
                f"{SCHALE_URL}images/student/lobby/Lobbyillust_Icon_{dev_name_}_01.png",
            ) as r:
                ret = await r.read()
        icon_img = Image.open(BytesIO(ret)).convert("RGBA")
        img.paste(icon_img, (left, top), icon_img)
        img.draw_text(
            (left, top + pic_h, left + icon_w, top + icon_h),
            name_,
            max_fontsize=25,
            min_fontsize=1,
        )

    task_li = []
    l = 0
    i = 0
    for stu in stu_li:
        if i == line_max_icon:
            i = 0
            l += 1
        task_li.append(draw_stu(stu["Name"], stu["DevName"], l, i))
        i += 1
    await asyncio.gather(*task_li)

    return MessageSegment.text(f"羁绊等级 {lvl} 时解锁L2D的学生有以下这些：") + MessageSegment.image(
        img.save("png")
    )
